<h3>Edit this document</h3>

<p>Go to <a href="https://github.com/fedora-infra/fedmsg/blob/develop/doc/{{ pagename }}.rst">{{ title }}</a>
on GitHub.
</p>
<p>Use the web interface to <b>fork</b> the repo, <b>edit</b> the file, and
<b>send a pull request</b>.</p>
<p>Your changes will be queued for review under project's
<a href="https://github.com/fedora-infra/fedmsg/pulls">Pull requests</a> tab on
Github.</p>
