{% extends "!page.html" %}
{% block footer -%}
{{ super() }}
<a href="https://github.com/fedora-infra/fedmsg">
  <img style="position: absolute; top: 0; right: 0; border: 0; max-width: 149px;"
  src="https://s3.amazonaws.com/github/ribbons/forkme_right_darkblue_121621.png"
  alt="Fork me on GitHub">
</a>
{%- endblock %}
